#ifndef HTML_DOCUMENT_H
#define HTML_DOCUMENT_H

/*************************************************************
*                                                            *
*       CGI C++ Library v0.29a                               *
*                                                            *
*       (c) 2005-2007 by Ighor Poteryakhin (ighor@ighor.ru)  *
*                                                            *
*       document.h                                           *
*                                                            *
*************************************************************/


#include "basic.h"
#include "const.h"

#include <iostream>
#include <fstream>
#include <sstream>

#include <ctime>
#include <string>
#include <deque>

namespace cgi
{
    enum codepage	{CP1251, CP866, KOI8R}; // add your codepages here and to cgi::document::scp()
    enum halignment	{HARIGHT, HALEFT, HACENTER, HAJUSTIFY};
    
    typedef std::deque<std::string> text;

    // implements methods to create and manipulate with html documents
    class document: public cgi::basic
    {
	public:
	    document();
	    document(codepage);
	    ~document();

	    void setcp(codepage);		// use this for clear contens
	    codepage cp();			// returns codepage as int
	    char* scp();			// returns codepage as text

	    void clear();
	    void print();
	    // header + nothing
	    void redirect(const char*);
	    void redirect(std::string&);
	    // server headers
	    void header(const char*);
	    void header(std::string);
	    // <title>
	    void title(const char*);
	    void title(std::string);
	    // <body background=...>
	    void background(const char*);
	    void background(std::string);
	    // <head>
	    void head(const char*);		// insert into <head> section </head>
	    void head(std::string);		// insert into <head> section </head>
	    // <body>
	    void body(int);			// insert into <body> section </body>
	    void body(const char*);		// insert into <body> section </body>
	    void body(std::string);		// insert into <body> section </body>
	    void body(const char*,halignment);	// insert into <body> section </body>
	    void body_file(const char*);	// insert file into <body> section </body>
	    void body_file(std::string&);	// insert file into <body> section </body>
	    // <hr> <br>
	    void br();				// insert <br> tag into <body> section </body>
	    void hr();				// insert <hr> tag
	    // <form>
	    void formstart(char*, char*);	// action, method
	    void forminput(char*, char*, char*);// type, name, value
	    void formstop();
	protected:
	    bool	_printed;

	private:
	    std::string	_title;
	    std::string _background;
	    text 	_header;
	    text 	_head;
	    text 	_body;
	    codepage 	_cp;
    };

    template <class what_class>
    cgi::document& operator<<(cgi::document& to,what_class what)
    {
        std::ostringstream s;
        s<<what;
        to.body(s.str());
        return to;
    };

}

#endif
